/*   contrib/m_opme.c
 *   Copyright (C) 2002 Hybrid Development Team
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   $Id: m_xlat.c,v 1.5 2003/03/11 18:22:49 newiceman Exp $
 */
#include "stdinc.h"
#include "tools.h"
#include "handlers.h"
#include "channel.h"
#include "client.h"
#include "ircd.h"
#include "numeric.h"
#include "s_log.h"
#include "s_serv.h"
#include "send.h"
#include "whowas.h"
#include "irc_string.h"
#include "hash.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"
#include "vchannel.h"
#include "hook.h"

#include <string.h>

static void m_xlat(struct Client *client_p, struct Client *source_p,
                    int parc, char *parv[]);

static char _koi[]="KOI\0";
static char _plain[]="PLAIN\0";
static char _lat[]="LAT\0";
static char _mac[]="MAC\0";
static char *_tbl[]={(char*)&_plain,(char*)&_koi,(char*)&_lat,(char*)&_mac};

static char _koitest[]	= "ţ\0";
static char _plaintest[]= "Ũ\0";
static char _lattest[] 	= "abvgdeezziiklmnoprstufhccss'y'euaABVGDEEZZIIKLMNOPRSTUFHCCSS'Y'EUA\0";
static char _mactest[]	= "ﰱ\0";
static char *_tbltest[]={(char*)&_plaintest,(char*)&_koitest,(char*)&_lattest,(char*)&_mactest};

static int iorecv(struct hook_io_data *);
static int flushlinebuf(struct hook_flush_linebuf_data *);
static void do_trans(char *, unsigned char, unsigned char, unsigned int);
static int show_codepage(struct hook_mfunc_data *);

struct Message xlat_msgtab = {
  "XLAT", 0, 0, 2, 0, MFLG_SLOW, 0,
  {m_unregistered, m_xlat, m_ignore, m_xlat}
};

void
_modinit(void)
{
  mod_add_cmd(&xlat_msgtab);
  hook_add_hook("iorecv", (hookfn *)iorecv); 
  hook_add_hook("flushlinebuf", (hookfn *)flushlinebuf); 
  hook_add_hook("doing_whois", (hookfn *)show_codepage);
}

void
_moddeinit(void)
{
  hook_del_hook("doing_whois", (hookfn *)show_codepage);
  hook_del_hook("flushlinebuf", (hookfn *)flushlinebuf); 
  hook_del_hook("iorecv", (hookfn *)iorecv); 
  mod_del_cmd(&xlat_msgtab);
}

char *_version = "$Revision: 1.5 $";


/*
** m_xlat
**      parv[0] = sender prefix
**      parv[1] = transtbl
*/
static void m_xlat(struct Client *client_p, struct Client *source_p,
                    int parc, char *parv[])
{
  unsigned char xlat_table;
  struct User *r;  
  if (!IsPerson(source_p))
    {
      sendto_one(source_p, ":%s NOTICE %s :XLAT works only for users", me.name,
                 parv[0]);
      return;
    }


	if(strcasecmp(parv[1],"PLAIN")==0) {
	    SetPlainXlat(source_p);
	} else if(strcasecmp(parv[1],"KOI")==0) {
	    SetKoiXlat(source_p);
	} else if(strcasecmp(parv[1],"LAT")==0) {
	    SetLatXlat(source_p);
	} else if(strcasecmp(parv[1],"MAC")==0) {
	    SetMacXlat(source_p);
	} else {
	    SetPlainXlat(source_p);
	}

      r = source_p->user;
      xlat_table = r->xlat_table;
      sendto_one(source_p, ":%s NOTICE %s :You translation table is [%s].", me.name,
                 parv[0],_tbl[xlat_table]);
      return;

    
}

static int iorecv(struct hook_io_data * data)
{
    struct User *r;
    unsigned char xlat_table;
    
    if (IsPerson(data->connection))
    {
	r = data->connection->user;
	xlat_table = r->xlat_table;
	if ((xlat_table != XLAT_LAT) && (xlat_table != XLAT_PLAIN))
	    do_trans(data->data, xlat_table, XLAT_PLAIN, data->len);
    }
    return 0;
}

static int flushlinebuf(struct hook_flush_linebuf_data * data)
{
    struct User *r;
    unsigned char xlat_table;

    if (IsPerson(data->connection))
    {
	r = data->connection->user;
	xlat_table = r->xlat_table;

	if (xlat_table != XLAT_PLAIN)
	    do_trans(data->line, XLAT_PLAIN, xlat_table, data->len);
    }
    
    return 0;
}

static void do_trans(char *dest, unsigned char from, unsigned char to, unsigned int len)
{
    unsigned int i, sym, lf, lt, ep;
    char *sfrom;
    char *sto;
    
    sfrom = _tbltest[from];
    sto = _tbltest[to];
   
    lf = strlen(sfrom);
    lt = strlen(sto);
    if (lf < lt) ep = lf; else ep = lt;
	 
    for (sym=0; sym < len; sym++)
    {
        for (i=0; i<ep; i++)
	{
	    if (dest[sym]==sfrom[i])
	    {
	        dest[sym] = sto[i];
	        break;
	    }
	}
    }
}

static int show_codepage(struct hook_mfunc_data *data)
{
  sendto_one(data->source_p, form_str(RPL_WHOISCODEPAGE), me.name,
           data->source_p->name, data->client_p->name, 
	   _tbl[data->client_p->user->xlat_table]);
  return 0;
}
